<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Entities;

use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calc\TaskInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Log\ProjectLogInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Log\TaskLogInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Tree\Treeble;
use Zeedhi\DevOps\Util\Events\Audit_with_id\AuditAble;
use Zeedhi\DevOps\Util\Events\Organization_with_id\OrganizationAble;


class Project
    extends \Zeedhi\DevOps\Model\Projectmanagement\Entities\Base\Project
    implements AuditAble, OrganizationAble, Treeble, TaskInterface, \Zeedhi\DevOps\Model\Util\LogicalDeleted,
    TaskCalendarInterface, ProjectLogInterface, TaskLogInterface
{
	const TABLE = "PROJECT";

    const IS_ACTIVE = 'A';
    const IS_ACTIVE_DEV_OPS = 'Y';
    const IS_NOT_ACTIVE = 'I';
    const IS_NOT_ACTIVE_DEV_OPS = 'N';
    const IS_NOTIFY_ISSUE = 'Y';
    const IS_NOT_NOTIFY_ISSUE = 'N';

    /** all CONTROL of PROJECT TYPE */
    const PROJECT_TYPE_ISSUE = ProjectType::TYPE_PROJECT_CONTROL_ISSUE;
    const PROJECT_TYPE_PRODUCT = ProjectType::TYPE_PROJECT_CONTROL_PRODUCT;
    const PROJECT_TYPE_STANDARD = ProjectType::TYPE_PROJECT_CONTROL_STANDARD;
    const PROJECT_TYPE_RELEASE = ProjectType::TYPE_PROJECT_CONTROL_RELEASE;
    const PROJECT_TYPE_SUB_RELEASE = ProjectType::TYPE_PROJECT_CONTROL_SUB_RELEASE;
    const PROJECT_TYPE_ERROR = ProjectType::TYPE_PROJECT_CONTROL_ERROR;
    const PROJECT_TYPE_MEETING = ProjectType::TYPE_PROJECT_CONTROL_MEETING;
    const PROJECT_TYPE_MEETING_OCCURRENCES = ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES;
    const PROJECT_TYPE_MODEL_RESTRICTED = ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED;
    const PROJECT_TYPE_MODEL_FREE = ProjectType::TYPE_PROJECT_CONTROL_MODEL_FREE;
    const PROJECT_TYPE_TEST = ProjectType::TYPE_PROJECT_CONTROL_TEST;

    const PROJECT_CONCLUDED_STATUS = TaskStatus::TASK_STATUS_CONTROL_CONCLUDED;
    const PROJECT_PAUSED_STATUS = TaskStatus::TASK_STATUS_CONTROL_NUMB;
    const PROJECT_IN_PROGRESS_STATUS = TaskStatus::TASK_STATUS_CONTROL_PROGRESS;
    const PROJECT_NOT_INITIALIZED_STATUS = TaskStatus::TASK_STATUS_CONTROL_PLANNING;
    const PROJECT_CANCELED_STATUS = TaskStatus::TASK_STATUS_CONTROL_CANCELED;
    const BRANCH_ERROR_MAIN = 'FIX_ERROR_PROJECT_';

    /** @var TaskInterface[] */
    private $children;
    /** @var \Closure */
	private $childrenClosure;
    private $needScheduleCalcDates = true;

    private $dateBeginMax;

    private $dateEndMin;
    /**
     * @var \Service\Calendar\Restriction\Strategy\RestrictionStrategy
     */
    private $restriction;

    protected $allCalendars;


    protected $durationPrecursor = 0;

	public function getTableName(){
		return self::TABLE;
	}

    public function getBranchErrorMain(){
        return self::BRANCH_ERROR_MAIN . $this->getId();
    }

	public function getTreeCode(){
		return $this->getTreeKey();
	}

	public function setTreeCode($key){
		$this->setTreeKey($key);
	}

	public function getTreeParent(){
		return $this->getParent();
	}

    public function setChildrenClosure(\Closure $childrenClosure){
        $this->childrenClosure = $childrenClosure;
    }
    
    public function directChildren(){
    	$subProjects = $this->getSubProjects();
    	$tasks = $this->getTasks();
    	$tasks = empty($tasks) ? array() : $tasks->getValues();
    	$subProjects = empty($subProjects)  ? array() : $subProjects->getValues();
        return array_merge($subProjects, $tasks);
    }

    /**
     * @return \Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calc\TaskInterface[]
     */
    public function children(){
        if ($this->children === null) {
            $this->children = $this->childrenClosure->__invoke();
        }
        return $this->children;
    }
    public function parent(){
        return $this->getTreeParent();
    }
    public function hasParent(){
        return !is_null($this->getTreeParent());
    }
    public function isParent(){
        return $this->hasChildren();
    }
    public function hasChildren(){
        return  count($this->directChildren()) > 0; 
    }
    
    public function setPlanned($time){
        $this->setTimePlanned($time);
    }
    public function setSpend($time){
        $this->setTimeSpent($time);
    }
    public function setRemaining($time){
        $this->setTimeRemaining($time);
    }    

    public function getPlanned(){
        $timePlanned= $this->getTimePlanned();
        if($this->isTimeLine()){
            $timePlanned = 0;
        }
        return $timePlanned;
    }
    public function getSpend(){
        $timeSpent = $this->getTimeSpent();
        if($this->isTimeLine()){
            $timeSpent = 0;
        }
        return $timeSpent;
    }
    public function getRemaining(){
        $timeRemaining = $this->getTimeRemaining();
        if($this->isTimeLine()){
            $timeRemaining = 0;
        }
        return $timeRemaining;
    }
    public function getEstimatedBeginDate(){

        return $this->getBeginDate();
    }
    public function setEstimatedBeginDate(\Datetime $estimatedBeginDate = NULL) {
        $this->setBeginDate($estimatedBeginDate);
    }
    public function getEstimatedEndDate(){
        return $this->getEndDate();
    }
    public function getEstimatedEndDateMaxParents(){
        if($this->getParent() && $this->getParent()->usesDataControl()){
            $return =  $this->getParent()->getEstimatedEndDateMaxParents();
        }else{
            $return = $this->getEstimatedEndDate();
        }
        return $return;
    }
    public function getEstimatedBeginDateMinParents(){
        if($this->getParent() && $this->getParent()->usesDataControl()){
            $return =  $this->getParent()->getEstimatedBeginDateMinParents();
        }else{
            $return = $this->getEstimatedBeginDate();
        }
        return $return;
    }
    public function setEstimatedEndDate(\Datetime $estimatedEndDate = NULL) {
        $this->setEndDate($estimatedEndDate);
    }


    public function getAllCalendars()
    {

        return $this->getParentsCalendar();
    }


    public function getRestriction()
    {
        return $this->restriction;
    }

    public function getParentsCalendar()
    {
        if(!$this->allCalendars){
            if ($this->getParent()) {
                $this->allCalendars = array_merge($this->getParent()->getParentsCalendar(), array($this->getCalendar()));
            } else {
                $this->allCalendars = array($this->getCalendar());
            }
        }
        return $this->allCalendars;
    }

    public function getDatesBeginAndEnd()
    {
        return $this->getRestriction()->getDatesBeginAndEnd();
    }

    /**
     * @return boolean
     */
    public function needScheduleCalcDates() {
        return $this->needScheduleCalcDates && $this->usesDataControl();
    }

    /**
     * @param mixed $needScheduleCalcDates
     */
    public function setNeedScheduleCalcDates($needScheduleCalcDates)
    {
        $this->needScheduleCalcDates = $needScheduleCalcDates;
    }

    /**
     * @return mixed
     */
    public function getDateBeginMax()
    {
        return $this->dateBeginMax;
    }

    /**
     * @param mixed $dateBeginMax
     */
    public function setDateBeginMax($dateBeginMax)
    {
        $this->dateBeginMax = $dateBeginMax;
    }

    /**
     * @return mixed
     */
    public function getDateEndMin()
    {
        return $this->dateEndMin;
    }

    /**
     * @param mixed $dateEndMin
     */
    public function setDateEndMin($dateEndMin)
    {
        $this->dateEndMin = $dateEndMin;
    }

    public function hasTaskRelationship()
    {
        return false;
    }

    /**
     * @param RestrictionStrategy $restriction
     * @return mixed|void
     */
    public function setRestriction($restriction)
    {
        $this->restriction = $restriction;
    }

    public function getTaskRelationship()
    {
        return null;
    }

    public function getTaskRelationshipInverse()
    {
        return array();
    }

    /**
     * @return boolean
     */
    public function isLogicalDeleted()
    {
        return $this->getStatus() == self::STATUS_LOGICAL_DELETED;
    }

    /**
     * @return int
     */
    public function getDurationPrecursor()
    {
        return $this->durationPrecursor;
    }

    /**
     * @param int $durationPrecursor
     */
    public function setDurationPrecursor($durationPrecursor)
    {
        $this->durationPrecursor = $durationPrecursor;
    }

    public function usesDataControl()
    {
        if(empty($this->projectType)){
            $return = true;
        }else{
            $return = $this->projectType->getUsetimeplanning() == \Service\Calendar\TaskCalendar::USES_DATA_CONTROL;
        }
        return $return;
    }

    public function isIssue()
    {
       return $this->getIsIssueProject() == 'Y';
    }

    public function isContainer()
    {
        return true;
    }

    public function getProject()
    {
        return $this;
    }

    public function getTask()
    {
        return null;
    }

    public function allowEndTask()
    {
        $projectType = $this->getProjectType();
        if($projectType){
            $allowEndTask = $projectType->getAllowendtask();
        }else{
            $allowEndTask = 'Y';
        }
        return $allowEndTask == 'Y';
    }

    //Functions used only for the calculation
    public function isTask()
    {
        return false;
    }

    public function getIsParent()
    {
        return 'Y';
    }
    /**
     * @param TaskInterface $child
     * @return bool
     */
    public function isValidateOfCalc($child)
    {
        return true;
    }

    public function isTimeLine()
    {
        return $this->getProjectType()->getExternaltype() == ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE;
    }

    public function getLatency()
    {
        return 0;
    }
    public function setLatency($latency)
    {

    }
    public function getPreviousTime()
    {
        return 0;
    }

    public function setProjectType(\Zeedhi\DevOps\Model\Projectmanagement\Entities\ProjectType $projectType = null) {
        if($projectType){
            parent::setProjectType($projectType);
        }
    }

    public function isActive() {
        return $this->getStatus() == self::IS_ACTIVE;
    }

    public function getStatusForDevOps()
    {
        $status = $this->getStatus();
        $projectStatusControl = $this->getProjectStatus()->getControl();
        $taskStatusActiveDevOps = array(
            TaskStatus::TASK_STATUS_CONTROL_IN_PROGRESS,
            TaskStatus::TASK_STATUS_CONTROL_IN_HOMOLOGATION,
        );
        if ($status == self::IS_ACTIVE && in_array($projectStatusControl,$taskStatusActiveDevOps)) {
            $status = self::IS_ACTIVE_DEV_OPS;
        } else {
            $status = self::IS_NOT_ACTIVE_DEV_OPS;
        }
        return $status;
    }
    public function isProduct(){
        return $this->projectType &&
        $this->projectType->getExternaltype() == ProjectType::TYPE_PROJECT_CONTROL_PRODUCT;
    }
    public function isMeeting(){
        return $this->projectType &&
        $this->projectType->getExternaltype() == ProjectType::TYPE_PROJECT_CONTROL_MEETING;
    }
    public function isMeetingOccurrences(){
        return $this->projectType &&
        $this->projectType->getExternaltype() == ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES;
    }

    public function deleteEntity()
    {
        $this->setStatus(self::STATUS_LOGICAL_DELETED);
    }

    public function setProjectStatus(TaskStatus $projectStatus = NULL)
    {
        if ($this->projectStatus) {
            if (!in_array($this->projectStatus->getControl(), array(self::PROJECT_CANCELED_STATUS, self::PROJECT_CONCLUDED_STATUS))) {
                $this->projectStatus = $projectStatus;
            }
        }else{
            $this->projectStatus = $projectStatus;
        }
    }
}